import type { SucceedUserInfoResponse, UserInfo } from '@/schemas';
import { type idAndNameSchema } from '@/schemas';
import { StorageSerializers, useLocalStorage } from '@vueuse/core';
import { defineStore } from 'pinia';
import { reactive, ref } from 'vue';
import { z } from 'zod';

type FullPermission = z.infer<typeof idAndNameSchema>;
export const useUserStore = defineStore('user', () => {
  const token = useLocalStorage<string | null>('token', null, {
    serializer: StorageSerializers.string
  });
  const userInfo = useLocalStorage<UserInfo | null>('user-info', null, {
    serializer: StorageSerializers.object
  });
  const permissions = reactive<FullPermission[]>([]);
  const isInitialized = ref(false);

  function updateUserInfo(userInfoResponse: SucceedUserInfoResponse) {
    userInfo.value = userInfoResponse.data;
  }

  function $reset() {
    token.value = null;
    userInfo.value = null;
    permissions.splice(0);
    isInitialized.value = false;
  }

  return {
    token,
    userInfo,
    permissions,
    isInitialized,
    updateUserInfo,
    $reset
  };
});
